/*

 ideserv.h - A mostly working PCLink server for IDEDOS 0.9x

 Written by
  Kajtar Zsolt <soci@c64.rulez.org>
 Win32 port by
  Josef Soucek <josef.soucek@ide64.org>
 Serial support by
  Borsanyi Attila <bigfoot@axelero.hu>
 Serial fixes by
  Silver Dream <silverdr@inet.com.pl>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

void timeout();
void seterror(int,int);

int relaxfiletypes;

volatile char tout;

enum e_modes {M_X1541, M_XE1541, M_XM1541, M_XA1541, M_PC64, M_PC64S, M_RS232, M_RS232S, M_USB, M_FIFO, M_ETHERNET};

static struct arguments_t
{
    int background;
    int priority;
    int relaxfiletypes;
    int debug;
    char *user;
    char *group;
    char *root;
    char *log;
    int hog;
    char *device;
    unsigned short int lptport;
    enum e_modes mode;
    struct in_addr sin_addr;
    unsigned char network;
} arguments;

#ifdef WIN32
#include <windows.h>

void err(int eval, const char *fmt, ...)
{
   MessageBox(NULL, fmt, NULL,MB_ICONERROR|MB_OK);
   exit(1);
}
#endif // WIN32

int check_trailer(char *mode, int usecrc) {
    int fr;
    if (arguments.mode == M_ETHERNET) return 0;
    if (usecrc) {
        if (io_ops->getb() < 0) return 1;
        if (crc_get()) {
	    seterror(23, 1);
            fprintf(lf, "CRC error while %s\n", mode);
            io_ops->getb();
            return 1;
        }
    }
    fr = io_ops->getb();if (fr < 0) return 1;
    if (fr != 0x5a) {
	fprintf(lf, "Frame error %02x while %s\n", fr, mode);
	return 1;
    }
    return 0;
}

int send_trailer(int usecrc) {
    if (arguments.mode == M_ETHERNET) return 0;
    if (usecrc) {
        if (io_ops->sendb(crc_get())) return 1;
    }
    if (io_ops->sendb(0x5a)) return 1;
    return io_ops->flush();
}

void timeout() {
    fprintf(lf,"Timeout.\n");
    io_ops->clean();
    tout=1;
}

void terminate() {
    if (arguments.mode != M_ETHERNET) io_ops->shutdown();
    if (lf) {
	fprintf(lf,"Terminated.\n");
	fclose(lf);
    }
    exit(0);
}

void seterror(int c,int i1) {
    char *msg="UNKNOWN ERROR";

    switch (c) {
    case 00:msg="OK";break;
    case 01:msg="FILES SCRATCHED";break;
    case 02:msg="PARTITION SELECTED";break;
    case 23:if (i1==0) msg="READ ERROR"; else msg="CRC ERROR";i1=0;break;
    case 26:msg="WRITE PROTECT ON";break;
    case 27:msg="ACCESS DENIED";break;
    case 31:msg="UNKNOWN COMMAND";break;
    case 30:
    case 33:
    case 34:msg="SYNTAX ERROR";break;
    case 39:msg="PATH NOT FOUND";break;
    case 62:msg="FILE NOT FOUND";break;
    case 63:msg="FILE EXISTS";break;
    case 72:msg="PARTITION FULL";break;
    case 73:msg="IDESERV 0.27B PCLINK";break;
    case 77:msg="SELECTED PARTITION ILLEGAL";break;
    }
    sprintf(errorbuff,"%02d, %s,%03d,000,000,000",c,msg,i1);
}

void errtochannel15(int i) {
    if (i)
        switch (errno) {
        case EISDIR:
        case ENOTDIR:
        case ENOENT:seterror(62,0);break;
        case EEXIST:seterror(63,0);break;
        case EPERM:
        case EACCES:seterror(27,0);break;
        case ENOTEMPTY:seterror(1,0);break;
        case ENOSPC:seterror(72,0);break;
        case EROFS:seterror(26,0);break;
        case EIO:seterror(23,0);break;
        default:seterror(80,errno);
        }
    else seterror(0,0);
}

